/**
  ******************************************************************************
  * File main.c
  * Description: Example of implementation of user bootloader (for Application Note AN2659)
  * Author: STMicroelectronics - MCD Prague Application Team
  * Version V2.0.0
  * Date 25/11/2010
  * Product: STM8S
  ******************************************************************************
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * COPYRIGHT 2008 STMicroelectronics
  ******************************************************************************
  */ 
  
//#pragma SRC is required for assembler code with raisonance
#pragma SRC
#include "main.h"
#include "stm8s.h"
#include "stm8s_gpio.h"
#include "stm8s_clk.h"
#include "stm8s_flash.h"
#include "stm8s_type.h"
#include "read_ID.h"
#include "CAN_Message.h"
#include "firmware_config.h"


#ifdef _RAISONANCE_
//needed by memcpy for raisonance
#include <string.h>
extern int __address__Mem_ProgramBlock; 
extern int __size__Mem_ProgramBlock; 
#endif /*_RAISONANCE_*/
extern int msgRec = 0;
extern u8 msgData1 = 0;
extern int appStart = 0;

//typedef FAR void (*)(void) TFunction;
typedef void ( *TFunction)(void);
/* conflicts
typedef enum {
     FLASH_MEMTYPE_PROG      = (u8)0xFD,
     FLASH_MEMTYPE_DATA      = (u8)0xF7 
} FLASH_MemType_TypeDef;
*/

//main application code (user reset) - init user code start - to interrupt table reset jump
const TFunction MainUserApplication = (TFunction)MAIN_USER_RESET_ADDR;
//address for GO command
TFunction GoAddress;

//Declare input buffer and its pointer 
u8 DataBuffer[130];
u8 *ReceivedData;
u8 DataBuff0;
u8 DataBuff1;

//Declare loaded into RAM
u8 RoutinesInRAM = 0;
u8 returnFW = 0;

void ProcessCommands(void);
u8 GT_Command_Process(void);
u8 RM_Command_Process(void);
u8 WM_Command_Process(void);
u8 GO_Command_Process(void);
u8 CheckTimeout(void);
u8 WriteBuffer(u8 FAR* DataAddress, u8 DataCount);
u8 WriteBufferFlash(u8 FAR* DataAddress, u8 DataCount, FLASH_MemType_TypeDef MemType);
void DeInitBootloader(void);
u8 Master_ident(u8 IDENT_byte);
void Transmit(u8 Data);
u8 Receive(u8* ReceivedData2);
u8 ReceiveAddress(void);
u8 ReceiveCount(u8 max, u8 enablechecksum);
void unlock_PROG(void);
void unlock_DATA(void);
void FLASH_ProgramOptionByte(u16 Address, u8 Data);
void Mem_ProgramBlock(u16 BlockNum, FLASH_MemType_TypeDef MemType, u8 *Buffer);
u32 CLK_GetClockFreq(void);
void ConfigClock();
void Config_IO_CAN();
u8 G_BOARDID[8]; 
u8 TransmitMailbox2;
u32 combine( u8 highest, u8 high, u8 mid, u8 low );
void delay(unsigned int cnt);
void gotoFirmware(void);
#define main_fw (*((void(*)(void))0xFA00))
 

//**************************************************************************
at 0x9080 void main_bl(void){
  sim();              // disable interrupts 
  RoutinesInRAM = 0;
  appStart = 0;  // Make sure interrupts go to Bootloader CAN_Message

	  // CAN Initialization from Firmware
      // initially configure all ports as output PP low slow
	// subsequent configuration should change ports to correct mode leaving
	// unused ports as output PP low slow
	// PC1 and PC2 should be left as inputs

   
    GPIO_Init(GPIOA, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port A
    GPIO_Init(GPIOB, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port B
    GPIO_Init(GPIOC, GPIO_PIN_ALL & (~(GPIO_PIN_1 | GPIO_PIN_2)), GPIO_MODE_OUT_PP_LOW_SLOW); // Port C
	GPIO_Init(GPIOD, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port D
    GPIO_Init(GPIOE, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port E
    GPIO_Init(GPIOF, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port F
    GPIO_Init(GPIOG, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port G
    GPIO_Init(GPIOH, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port H
    GPIO_Init(GPIOI, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port I
    
	GPIO_WriteHigh(GPIOC, GPIO_PIN_4); 	

	// Try setting all unused ports (that don't have a pin out on 48-pin MCU to PP Low Slow
	// this makes no difference to power consumption
    GPIO_Init(GPIOI, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port I
	GPIO_Init(GPIOH, GPIO_PIN_ALL, GPIO_MODE_OUT_PP_LOW_SLOW); // Port H
    GPIO_Init(GPIOC, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_SLOW);   // Port C

	ConfigClock();
	
	//CLK->HSITRIMR = ((s8)  -1);
	
	Config_IO_CAN();
	
	// Initialise G_BOARDID array to 0
	// Must read ID before configuring CAN - board ID needed to set up arbitration.
	G_BOARDID[0] = 0x0;
	G_BOARDID[1] = 0x0;
	G_BOARDID[2] = 0x0;
	G_BOARDID[3] = 0x0;
	G_BOARDID[4] = 0x0;
	G_BOARDID[5] = 0x0;
	G_BOARDID[6] = 0x0;
	G_BOARDID[7] = 0x0;
		

    
    OWReadID();
	
	CAN_InitTransmit();
    
    
    	// Configure ports for ADCs
        
	//Config LEDs
	GPIO_Init(GPIOE, (GPIO_PIN_1| GPIO_PIN_2), GPIO_MODE_OUT_OD_LOW_SLOW);
    SetLEDs(0x00);
  
    rim(); //Allow interrupts
  //wait for identification
    if(!Master_ident(IDENT))  
    {
    Transmit(NACK);
		//identification FAILED 
    DeInitBootloader();     // then set back all microcontroller changes to reset values
    //reset stack pointer (lower byte - because compiler decreases SP with some bytes)
    #pragma ASM
    LDW X,SP
    LD A,0FFH
    LD XL,A
    LDW SP,X
    JPF 09000H  
    #pragma ENDASM 

  }
  //identification SUCESS - copy routines to RAM
  RoutinesInRAM = 1;
  unlock_PROG();
  unlock_DATA();
  FLASH_SetProgrammingTime(FLASH_PROGRAMTIME_STANDARD);
	//send ACK to host
  Transmit(ACK);
  do
  {
    //process commands from host
    ProcessCommands();

    //received GO command - set back all microcontroller changes to reset values
    DeInitBootloader();

    //jump to GO address
    GoAddress();
  }while (1);
}//main

void ProcessCommands(void){
  u8 result;
  u8 wait;
	do
  {
		wait = 8;
		while(wait)
			wait --;
		
	ReceivedData = DataBuffer;
    //receive data 1-st byte
    if(!Receive(ReceivedData++))
      continue;
  
    //receive data 2-nd byte
    if(!Receive(ReceivedData++))
      continue;
  
	DataBuff0 = DataBuffer[0]; //Complement Bytes
	DataBuff1 = DataBuffer[1];

    //check if command bytes are complements
    //if(DataBuffer[0] != (u8)(~DataBuffer[0]++))
	if(DataBuff0 != (u8)(~DataBuff1))
    {
      Transmit(NACK);
      continue;
    }
  
    //parse commands
    Transmit(ACK);
    switch(DataBuffer[0])
    {
      case (GT_Command): result = GT_Command_Process(); break;
      case (RM_Command): result = RM_Command_Process(); break;
      //case (GO_Command): result = GO_Command_Process(); break;
      case (WM_Command): result = WM_Command_Process(); break;
    }
  }while( (!result) || (DataBuff0 != GO_Command) ); //until GO command received
  gotoFirmware();
}//ProcessCommands

u8 GT_Command_Process(void){
  //send bytes number (Version+commands)=N
  Transmit(Num_GT_Command);
  
  //Send the BL Version Number
  Transmit(Version_Number);
  
  //Send GET command
  Transmit(GT_Command);
  
  //Send Read Memory command
  Transmit(RM_Command);
  
  //Send GO command
  Transmit(GO_Command);
  
  //Send Write Memory command
  Transmit(WM_Command);
  
  //Send Erase Memory command
  Transmit(EM_Command);
  
  //Send Erase Memory command
  Transmit(ACK);
  
  return 1;
}//GT_Command

u8 RM_Command_Process(void){
  u8 i;
	u8 FAR* DataAddress;
  u8 Checksum;
  //receive address
  if(!ReceiveAddress())
  {
    Transmit(NACK);// if not valid - NACK
    return 0;
  }
    
  //update address to read from
  DataAddress = *(u8 FAR**)(&DataBuffer[N_ADDR_3]);
  
  //receive count
  if(!ReceiveCount(0xFF, 1))
  {
    Transmit(NACK);// if not valid - NACK
    return 0;
  }
   
  //send data
  Checksum = 0;
  for(i=0; i<=DataBuffer[N_DATACOUNT]; i++)
  {
    Checksum ^= DataAddress[i];
    Transmit(DataAddress[i]);
  }
  return 1;
}//RM_Command

u8 GO_Command_Process(void){
	u32 Address32;
	u16 Address16;
  //receive address
  if(!ReceiveAddress())
  {
    Transmit(NACK);// if not valid - NACK
    return 0;
  }
  //update GO address 
  Address32 = *(u32*)(&DataBuffer[N_ADDR_3]);
	Address16 = (u16)(Address32 >> 8);
  GoAddress = (TFunction)Address16;
  return 1;
}//GO_Command

u8 WM_Command_Process(void){
  u8 i;
	u8 FAR* DataAddress;
  u8 Checksum;
  u8 DataCount;
  u8 ChecksumByte;
  int ierror = 0;
  
  //receive address
  if(!ReceiveAddress())
  {
    Transmit(NACK);// if not valid - NACK
    return 0;
  }
  
  //update address to write to
  DataAddress = *(u8 FAR**)(&DataBuffer[N_ADDR_3]);
  

  //receive count
   if(!ReceiveCount(BLOCK_SIZE-1, 0))
  {
    Transmit(NACK);// if not valid - NACK
    return 0;
  }  
  
  //receive data
  DataCount = DataBuffer[N_DATACOUNT]; // init count
  Checksum = DataCount;                //init checksum 
  Checksum += DataBuffer[N_ADDR_2];
  Checksum += DataBuffer[N_ADDR_1];
  
  for(i=0; i<DataCount; i++)
  {
    if(!Receive(&DataBuffer[i]))
		{
        ierror = 1;
		}
		//calculate checksum from received bytes
    Checksum += DataBuffer[i];
  }
  //receive checksum byte
  if(!Receive(&ChecksumByte))
  {
		ierror = 1;
  }
  //check checksum
  Checksum = ~Checksum;
  Checksum++;
  if(ChecksumByte != Checksum)
  {
    ierror = 1;
  }
  
  //if some error occured during data reception
  if (ierror == 1)
  {

    Transmit(NACK); //send error
    return 0;    
  }
  
  //write buffer to memory
  if (!WriteBuffer(DataAddress, DataCount+1))
  {
    Transmit(NACK); //send error
    return 0;    
  }
  //confirm correct writing
  Transmit(ACK);
  return 1;
}//WM_Command

u8 CheckAddress(u32 DataAddress){
  //for Flash
  if ((DataAddress >= FLASH_START) && (DataAddress <= FLASH_END))
    return 1;
  else
    //for EEPROM
    if ((DataAddress >= EEPROM_START) && (DataAddress <= EEPROM_END))
      return 1;
    else
      //for RAM
      if ((DataAddress >= RAM_START) && (DataAddress <= RAM_END))
        return 1;
      else
        //for Option bytes
        if ((DataAddress >= OPTION_START) && (DataAddress <= OPTION_END))
          return 1;
  return 0;
}//CheckAddress

u8 WriteBuffer(u8 FAR* DataAddress, u8 DataCount){
  u8 i;
  
  //for Flash
  if (((u32)DataAddress >= FLASH_START) && (((u32)DataAddress + DataCount - 1) <= FLASH_END))
    return WriteBufferFlash(DataAddress, DataCount, FLASH_MEMTYPE_PROG);
    
  //for EEPROM
  if (((u32)DataAddress >= EEPROM_START) && (((u32)DataAddress + DataCount - 1) <= EEPROM_END))
    return WriteBufferFlash(DataAddress, DataCount, FLASH_MEMTYPE_DATA);
    
  //for RAM
  if (((u32)DataAddress >= RAM_START) && (((u32)DataAddress + DataCount - 1) <= RAM_END))
  {
    for(i=0; i<DataCount; i++)
      DataAddress[i] = DataBuffer[i];
    return 1;
  }
  
  //for Option bytes
  if (((u32)DataAddress >= OPTION_START) && (((u32)DataAddress + DataCount - 1) <= OPTION_END))
  {
    for(i=0; i<DataCount; i++)
    {
       FLASH_ProgramOptionByte((u32)(&DataAddress[i]), DataBuffer[i]);
    }
    return 1;
  }
  
  //otherwise fail
  return 0;
}//WriteBuffer

u8 WriteBufferFlash(u8 FAR* DataAddress, u8 DataCount, FLASH_MemType_TypeDef MemType){
  u32 Address = (u32) DataAddress;
  u8 * DataPointer = DataBuffer;
  u32 Offset;
  //set offset according memory type
  if(MemType == FLASH_MEMTYPE_PROG)
    Offset = FLASH_START;
  else
    Offset = EEPROM_START;
  //program beginning bytes before words
  while((Address % 4) && (DataCount))
  {
	  FLASH_ProgramByte(Address, *DataPointer);
	  Address++;
	  DataPointer++;
	  DataCount--;
	}
  //program beginning words before blocks
	while((Address % BLOCK_BYTES) && (DataCount >= 4))
  {
	  u32 DataWord = combine(*(DataPointer),*(DataPointer+1),*(DataPointer+2), *(DataPointer+3));
	  FLASH_ProgramWord(Address, DataWord);
	  Address+= 4;
	  DataPointer+=4;
	  DataCount-=4;
  }
  //program blocks
  while(DataCount >= BLOCK_BYTES)
  {
    Mem_ProgramBlock((Address - Offset)/BLOCK_BYTES, MemType, DataPointer);
    Address    += BLOCK_BYTES;
    DataPointer+= BLOCK_BYTES;    
    DataCount  -= BLOCK_BYTES;
  }
  
  //program remaining words (after blocks)
  while(DataCount >= 4)
  {
	  u32 DataWord = combine(*(DataPointer),*(DataPointer+1),*(DataPointer+2), *(DataPointer+3));
	  FLASH_ProgramWord(Address, DataWord);
	  Address+= 4;
	  DataPointer+=4;
	  DataCount-=4;
  }
  
  //program remaining bytes (after words)
  while(DataCount)
  {
      FLASH_ProgramByte(Address, *DataPointer);
	  Address++;
	  DataPointer++;
      DataCount--;
  }
  
  return 1;
}//WriteBufferFlash

void DeInitBootloader(void){
  if(RoutinesInRAM)
  {
    /* Lock program memory */
    FLASH->IAPSR = ~0x02;
    /* Lock data memory */
    FLASH->IAPSR = ~0x08;
  }
	
  /* DeInit I2C */	
	I2C->CR1 = 0x00;
  I2C->CR2 = 0x00;
  I2C->FREQR = 0x00;
  I2C->OARL = 0x00;
  I2C->OARH = 0x00;
  I2C->ITR = 0x00;
  I2C->CCRL = 0x00;
  I2C->CCRH = 0x00;
  I2C->TRISER = 0x02;
	
}//DeInitBootloader

void Transmit(u8 Data){
	u8 TxData[8];
	u32 i;
    for(i=0; i<8; i++){
        TxData[i] = 0x0;
    }
	TxData[0] = Data;
	TxData[1] = 0x11;
	TxData[2] = 0x22;
	CAN_SendData(&TxData[0]);
	
	}//Transmit

u8 Receive(u8* ReceivedData2){
  	do{
			
	}while(msgRec == 0);
	*ReceivedData2 = msgData1;
	msgRec = 0;
	return 1;
}//Receive

u8 Master_ident(u8 IDENT_byte){
	u8 master_ident;
 	Receive(&master_ident);
  if (master_ident == IDENT_byte) 
	{
		return 1;	
  } 
	return 0;
}//Master_ident

u8 ReceiveAddress(void){
  u8 result = 1;
  
  //receive address (4 bytes : Byte3: MSB , Byte6: LSB)
  if(!Receive(ReceivedData++))
    result = 0;
  if(!Receive(ReceivedData++))
    result = 0;
  if(!Receive(ReceivedData++))
    result = 0;
  if(!Receive(ReceivedData++))
    result = 0;
    
  //receive checksum (from 4 bytes : Byte3: MSB - Byte6: LSB)
  if(!Receive(ReceivedData++))
    result = 0;
  
  //check checksum
  if(*(--ReceivedData) ^ *(--ReceivedData) ^ *(--ReceivedData) ^ *(--ReceivedData) ^ *(--ReceivedData))
    result = 0;
  //set back data pointer
  ReceivedData += 5;
  
  //check if address is valid
  //if(!CheckAddress(*((u32*) &DataBuffer[N_ADDR_4]) ))
    //result = 0;
  
  //if some error occured
  if (!result)
    return 0;
  
  //confirm correct reception
  Transmit(ACK);
  return 1;
}//ReceiveAddress

u8 ReceiveCount(u8 max, u8 enablechecksum){
  bool result = 1;
  
  //receive count
  if(!Receive(ReceivedData))
    result = 0;
    
  //check maximum value
  if(!((*ReceivedData++) <= max))
    result = 0;
 
 
 //receive checksum if enabled
  if (enablechecksum)
  {
    //receive checksum
    if(!Receive(ReceivedData++))
      result = 0;
      
    //check checksum
    if((*(--ReceivedData) ^ *(--ReceivedData)) != 0xFF)
      result = 0;
    //set back data pointer
    ReceivedData += 2;
  }
	
	//confirm correct reception - if required
  if (enablechecksum)
    Transmit(ACK);
  
  return result;
}//ReceiveCount

void unlock_PROG(void){
  //Unlock PROG memory
  FLASH->PUKR = 0x56;
  FLASH->PUKR = 0xAE;
  }
	
void unlock_DATA(void) {
 //Unlock DATA memory
	FLASH->DUKR = 0xAE; /* Warning: keys are reversed on data memory !!! */
  FLASH->DUKR = 0x56;
}

#ifdef _IAR_
#pragma location = "FLASH_CODE"
#endif /* _IAR_ */

#ifdef _RAISONANCE_
void Mem_ProgramBlock(u16 BlockNum, FLASH_MemType_TypeDef MemType, u8 *Buffer) inram
#else
void Mem_ProgramBlock(u16 BlockNum, FLASH_MemType_TypeDef MemType, u8 *Buffer)
#endif /*_RAISONANCE_*/
{
    u16 Count = 0;
    u32 StartAddress = 0;
    u16 timeout = (u16)0x6000;
 
 /* Set Start address wich refers to mem_type */
    if (MemType == FLASH_MEMTYPE_PROG)
    {
        StartAddress = FLASH_START;
    }
    else
    {
        StartAddress = EEPROM_START;
    }

    /* Point to the first block address */
    StartAddress = StartAddress + ((u32)BlockNum * BLOCK_SIZE);

    /* Standard programming mode */ 
    FLASH->CR2 |= (u8)0x01;
    FLASH->NCR2 &= (u8)~0x01;
    
    /* Copy data bytes from RAM to FLASH memory */
    for (Count = 0; Count < BLOCK_SIZE; Count++)
    {
        *((PointerAttr u8*)StartAddress + Count) = ((u8)(Buffer[Count]));
    }

 #if defined (STM8S208) || defined(STM8S207) || defined(STM8S105)
    if (MemType == FLASH_MEMTYPE_DATA)
    {
        /* Waiting until High voltage flag is cleared*/
        while ((FLASH->IAPSR & 0x40) != 0x00 || (timeout == 0x00))
        {
            timeout--;
        }
    }
 #endif /* STM8S208, STM8S207, STM8S105 */
}

void ConfigClock(void)
{
	
		//CONFIG CLOCK
    GPIO_Init(GPIOE, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_SLOW);
       
    CLK_HSIPrescalerConfig(CLK_CKDIVR_HSIDIV);
    
    CLK->PCKENR2 |= CLK_PCKENR2_CAN;
    
    CLK_CCOCmd(ENABLE);
	
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER1, DISABLE);
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER2, DISABLE);
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER3, DISABLE);
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER4, DISABLE);
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_I2C, DISABLE);   
	//CLK_PeripheralClockConfig(CLK_PERIPHERAL_SPI, DISABLE);   
	//CLK_PeripheralClockConfig(CLK_PERIPHERAL_USART, DISABLE); 
	//CLK_PeripheralClockConfig(CLK_PERIPHERAL_LINUART, DISABLE);    
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_UART1, DISABLE); 
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_UART2, DISABLE); 
	CLK_PeripheralClockConfig(CLK_PERIPHERAL_AWU, DISABLE);
	//CLK_PeripheralClockConfig(CLK_PERIPHERAL_ADC, DISABLE); 
	//CLK_PeripheralClockConfig(CLK_PERIPHERAL_CAN, DISABLE); 
	
}

void Config_IO_CAN(void)
{
    // CAN_TX
    GPIO_Init(GPIOG, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST);
    // CAN_RX
    //GPIO_Init(GPIOG, GPIO_PIN_1, GPIO_MODE_IN_PU_NO_IT);
    GPIO_Init(GPIOG, GPIO_PIN_1, GPIO_MODE_IN_PU_IT);
    // Configure the CAN transceiver in active mode
    GPIO_Init(GPIOC, GPIO_PIN_3, GPIO_MODE_OUT_PP_LOW_FAST);	
}

void delay(unsigned int cnt)
{
    unsigned short loop;

    /* Decrement counter until it is zero */
    while (cnt--)
        for(loop = 921; loop; loop--)   /* Wait for approximatively 1 ms */ 
            _nop_(); 
}

u32 combine( u8 highest, u8 high, u8 mid, u8 low )
{
   return ( ((u32)highest << 24) |((u32)high << 16 ) | ( (u32)mid << 8 ) | ((u32)low ));
}

void gotoFirmware(void)
{

    /*
	appStart = 1;
	main_fw();
    sim();
    */

    /*
    #pragma ASM //reset stack pointer
    LDW X,SP
    LD A,0FFH
    LD XL,A
    LDW SP,X
    JPF 0FA00H  
    #pragma ENDASM 
    */
    //_jmp_((_fctptr_t)0xFA00);
}

//**************************************************************************

