#ifndef __FIRMWARE_CONFIG_H
#define __FIRMWARE_CONFIG_H

// Firmware version
#define fwareVint 1     // i.e. version 1.X
#define fwareVflo 3     // i.e. version X.2

extern u8 TestPulserFlag;

/*Values of first byte in CAN message*/
#define COMMAND_PD  0xAA
#define COMMAND_PU  0xAB
#define COMMAND_ADC 0xBB
#define COMMAND_DAC 0xCC
#define COMMAND_IAP 0xDD
#define COMMAND_TP  0xEE
#define COMMAND_RC  0xFF
#define COMMAND_HV  0x99
#define COMMAND_ID  0x88
#define COMMAND_LED 0x77
#define COMMAND_VER 0x66
#define COMMAND_STAT 0x11
#define READ_MVB	0x00
#define READ_MVT	0x01
#define READ_VDY01	0x02
#define READ_VCATH	0x03
#define READ_VDAC	0x04
#define READ_TEMP 	0x05
#define READ_CURR 	0x06
#define TP_FIRE		0x00
#define TP_ENBSYNC	0x01
#define TP_DISSYNC	0x02
#define STAT_READ   0x01
#define STAT_WRITE  0x02

/*Port values*/
#define BOARDID_PORT (GPIOE)		
#define TP_PORT      (GPIOG)
#define DAC_PORT     (GPIOC)
#define LEDS_PORT    (GPIOE)
#define CAN_PORT_G   (GPIOG)
#define CAN_PORT_C   (GPIOC)
#define MON_PORT	 (GPIOB)
#define ENB_PORT	 (GPIOD)

/*Pin values*/
#define BOARDID_PIN 	(GPIO_PIN_4)
#define ENB_SYNCTP_PIN  (GPIO_PIN_6)
#define FIRE_TP_PIN     (GPIO_PIN_5)
#define DAC_CS_PIN     	(GPIO_PIN_4)
#define LED_GREEN_PIN	(GPIO_PIN_1)
#define LED_RED_PIN   	(GPIO_PIN_2)
#define CAN_STBY_PIN   	(GPIO_PIN_3)
#define CAN_TX_PIN   	(GPIO_PIN_0)
#define CAN_RX_PIN   	(GPIO_PIN_1)
// Cockroft Walton enable/disable pins
#define ENB_MVT_PIN	(GPIO_PIN_7)
#define ENB_MVB_PIN	(GPIO_PIN_6)
#define JAM_HVB_PIN	(GPIO_PIN_5)
#define ENB_HVB_PIN	(GPIO_PIN_4)
#define JAM_HVT_PIN	(GPIO_PIN_3)
#define ENB_HVT_PIN	(GPIO_PIN_2)
// ADCs for monitoring:
#define MON_TEMP_PIN		(GPIO_PIN_6)
#define MON_CURR_PIN		(GPIO_PIN_5)
#define	MON_VDAC_PIN		(GPIO_PIN_4)
#define	MON_VCATH_PIN		(GPIO_PIN_3)
#define	MON_VDY01_PIN		(GPIO_PIN_2)
#define	MON_MVT_PIN		(GPIO_PIN_1)
#define	MON_MVB_PIN		(GPIO_PIN_0)

/*Channel values*/
#define MON_MVB_CHANNEL	(ADC2_CHANNEL_0)
#define MON_MVT_CHANNEL	(ADC2_CHANNEL_1)
#define MON_VDY01_CHANNEL	(ADC2_CHANNEL_2)
#define MON_VCATH_CHANNEL	(ADC2_CHANNEL_3)
#define MON_VDAC_CHANNEL	(ADC2_CHANNEL_4)
#define MON_CURR_CHANNEL	(ADC2_CHANNEL_5)
#define MON_TEMP_CHANNEL	(ADC2_CHANNEL_6)

/*Schmitt trigger values*/
#define MON_MVB_SCHMITTTRIG_CHANNEL		(ADC2_SCHMITTTRIG_CHANNEL0)
#define MON_MVT_SCHMITTTRIG_CHANNEL		(ADC2_SCHMITTTRIG_CHANNEL1)
#define MON_VDY01_SCHMITTTRIG_CHANNEL	(ADC2_SCHMITTTRIG_CHANNEL2)
#define MON_VCATH_SCHMITTTRIG_CHANNEL	(ADC2_SCHMITTTRIG_CHANNEL3)
#define MON_VDAC_SCHMITTTRIG_CHANNEL	(ADC2_SCHMITTTRIG_CHANNEL4)
#define MON_CURR_SCHMITTTRIG_CHANNEL (ADC2_SCHMITTTRIG_CHANNEL5)
#define MON_TEMP_SCHMITTTRIG_CHANNEL (ADC2_SCHMITTTRIG_CHANNEL6)

#endif