

#include <intrist7.h>   /* Defines the _nop_() intrinsic function */
#include <stdio.h>     /* contains sprintf */
#include "stm8s_conf.h"
#include "stm8s_can.h"
#include "stm8s_flash.h"
#include "firmware_config.h"
#include "CAN_Message.h"
#include "read_ID.h"
#include "config_commands.h"
#include "stm8s_gpio.h"

/*****************************************************************************
 * Can handling functions.
 */

volatile CanRxMsg _RxMessage;
volatile CanTxMsg _TxMessage;

volatile CAN_InitTypeDef        _CAN_InitStructure;
volatile CAN_FilterInitTypeDef  _CAN_FilterInitStructure;

extern int msgRec;
extern u8 msgData1;

void CAN_InitTransmit(void)
{
	extern u8 G_BOARDID[8];	
	
	// G_BOARDID[1] contains EXID[7:0]
	// G_BOARDID[2] contains EXID[15:8]
	// G_BOARDID[3] contains EXID[23:16]
	u8 idBits_6_0_X;			
	u8 idBits_14_7;
	u8 idBits_20_18_X_17_15;
	u8 idBits_X_23_21;          
	u8 x; 
	u8 y; 
	u8 z; 
	u8 a; 
	u8 b; 
	u8 c; 
	u8 d; 
	u8 e; 
	u8 f;  
	x = G_BOARDID[3];
	y = G_BOARDID[2];
	z = G_BOARDID[1];
	a = (x >> 5) & 0x07;
	b = (x >> 2) & 0x07;
	c = ( (x << 1) & 0x06 ) | ( (y >> 7) & 0x01 );
	d = ( (y << 1) & 0xFE ) | ( (z >> 7) & 0x01 );
	e = (z << 1) & 0xFE;
	f = ( ( (b << 5) & 0xE0 ) | c ) | 0x08;
	
	idBits_6_0_X = e;
	idBits_14_7 = d;
	idBits_20_18_X_17_15 = f;
	idBits_X_23_21 = a;

    /* CAN register init */
    CAN_DeInit();
    CAN_StructInit(&_CAN_InitStructure);

    /* Initialize for a 50kHz baud rate */
    /* CAN cell init */
	_CAN_InitStructure.ABOM=ENABLE;   //d.w.b. Recover from Bus-Off state if too many errors.
    _CAN_InitStructure.TXFP=DISABLE;
    _CAN_InitStructure.SJW=CAN_SJW_4tq;
    _CAN_InitStructure.BS1=CAN_BS1_16tq;
    _CAN_InitStructure.BS2=CAN_BS2_3tq;
    _CAN_InitStructure.Prescaler=2;	  // this prescaler works with 1MHz clock
    CAN_Init(&_CAN_InitStructure);

    /* CAN filter init */
    //---------------------------------------------------------------
	// Set up first filter to filter on chip ID:
	// NB. Must have RTR bit = 0 and IDE = 1
    //---------------------------------------------------------------
	
    _CAN_FilterInitStructure.CAN_FilterNumber=0;
    _CAN_FilterInitStructure.CAN_FilterMode=((u8)0x03);  // id list mode 
    _CAN_FilterInitStructure.CAN_FilterScale=CAN_FilterScale_32bit; // make sure CAN_FilterScale_32bit = 0x03
    	
	// 32-bit filter 1 (ext ID = chip id):
    _CAN_FilterInitStructure.CAN_FilterIdLow1=idBits_X_23_21;
    _CAN_FilterInitStructure.CAN_FilterIdLow2=idBits_20_18_X_17_15;
    _CAN_FilterInitStructure.CAN_FilterIdHigh1=idBits_14_7;
    _CAN_FilterInitStructure.CAN_FilterIdHigh2=idBits_6_0_X;
	
    _CAN_FilterInitStructure.CAN_FilterMaskIdLow1=idBits_X_23_21;
    _CAN_FilterInitStructure.CAN_FilterMaskIdLow2=idBits_20_18_X_17_15;
    _CAN_FilterInitStructure.CAN_FilterMaskIdHigh1=idBits_14_7;
    _CAN_FilterInitStructure.CAN_FilterMaskIdHigh2=idBits_6_0_X;
	
    _CAN_FilterInitStructure.CAN_FilterActivation=ENABLE;
    CAN_FilterInit(&_CAN_FilterInitStructure);
	
    //---------------------------------------------------------------
	
    //---------------------------------------------------------------
	// Set up a second filter to also accept messages with ID 0x0: 
	// NB. Must have RTR bit = 0 and IDE = 1
    //---------------------------------------------------------------
	
    _CAN_FilterInitStructure.CAN_FilterNumber=1;
    _CAN_FilterInitStructure.CAN_FilterMode=((u8)0x03);  //id list mode 
    _CAN_FilterInitStructure.CAN_FilterScale=CAN_FilterScale_32bit; // make sure CAN_FilterScale_32bit = 0x03

	// 32-bit filter 1 (ext ID = 0x0):
    _CAN_FilterInitStructure.CAN_FilterIdLow1=0x00;
    _CAN_FilterInitStructure.CAN_FilterIdLow2=0x08;
    _CAN_FilterInitStructure.CAN_FilterIdHigh1=0x00;
    _CAN_FilterInitStructure.CAN_FilterIdHigh2=0x00;

    _CAN_FilterInitStructure.CAN_FilterMaskIdLow1=0x00;
    _CAN_FilterInitStructure.CAN_FilterMaskIdLow2=0x08;
    _CAN_FilterInitStructure.CAN_FilterMaskIdHigh1=0x00;
    _CAN_FilterInitStructure.CAN_FilterMaskIdHigh2=0x00;
	
    _CAN_FilterInitStructure.CAN_FilterActivation=ENABLE;
    CAN_FilterInit(&_CAN_FilterInitStructure);
	
    //---------------------------------------------------------------

    /* Request leave initialisation */
    CAN_MCR &= (u8)(~CAN_MCR_INRQ);
    /* Wait the acknowledge */
    while (CAN_MSR & CAN_MCR_INRQ);
    /* ...and check acknowledged */
    /*if ((CAN_MSR & CAN_MSR_INAK) == CAN_MSR_INAK)
    {
        InitStatus = CANINITFAILED;
    }
    */
    while(CAN_MessagePending()>=1); //wait til mbx is empty
}

void CAN_TransmitPowerDown(void){
	// Pull CAN_STBY pin high - won't be able to read CAN messages
	GPIO_WriteHigh( GPIOC, CAN_STBY_PIN );
	// enable external interrupts on PC1 so that
	// base can be woken back up by activity on CAN_RX line
	GPIOC->CR2 |= GPIO_PIN_1;
}
void CAN_TransmitPowerUp(void){
	// Pull CAN_STBY pin low - MCU will now be able to read/write
	// CAN messages
	GPIO_WriteLow( GPIOC, CAN_STBY_PIN );
	// disable external interrupts on PC1 
	GPIOC->CR2 &= ~GPIO_PIN_1;
}

at 0xB000 void CAN_GetDataBL(void)
{
        
	extern u8 G_BOARDID[8];
	//extern u8 TestPulserFlag;
    u8 TxData[8];
    u32 i;

    //CanRxMsg RxMessage;
    u8 transmitMailbox;
    u16 conversionValue;
	u8 hvStatus;
    	
	// initialise data array for can message
    for(i=0; i<8; i++){
        TxData[i] = 0x0;
    }

    /* receive */
    _RxMessage.StdId=(u16)(0x0000);
    _RxMessage.IDE=CAN_ID_EXT;
    _RxMessage.DLC=0;
    _RxMessage.Data[0]=0x00;
    _RxMessage.Data[1]=0x00;
    _RxMessage.Data[2]=0x00;
    _RxMessage.Data[3]=0x00;
    _RxMessage.Data[4]=0x00;
    _RxMessage.Data[5]=0x00;
    _RxMessage.Data[6]=0x00;
    _RxMessage.Data[7]=0x00;

    while( ( CAN_MessagePending() ) < 1 );

    CAN_Receive(&_RxMessage);
	
    //Start In-Application programming
    if ((u8)(_RxMessage.Data[0]) == COMMAND_IAP){
		msgData1 = _RxMessage.Data[1];
		msgRec = 1;   
    }
	
	else if ((u8)(_RxMessage.Data[0]) == COMMAND_LED ){
        if ((u8)(_RxMessage.Data[1]) == 0x00 ){  
            // For bases - turn red led on
            //SetLEDs(0x00);
            SetLEDs(LED_GREEN_PIN);
        }
        else if((u8)(_RxMessage.Data[1]) == 0x01 ){  
            // For bases - turn green led on
            //SetLEDs(0x00);
            SetLEDs(LED_RED_PIN);
        }
        else if((u8)(_RxMessage.Data[1]) == 0x02 ){  
            // For bases - turn LED off
            SetLEDs(0x00);
        }
        else if((u8)(_RxMessage.Data[1]) == 0x03 ){  
            // For bases - turn both LEDs on
            //SetLEDs(0x00);
            SetLEDs(LED_RED_PIN | LED_GREEN_PIN);
        }
    }

    else if ((u8)(_RxMessage.Data[0]) == COMMAND_STAT){
        if ((u8)(_RxMessage.Data[1]) == 0x01 ){
        
            TxData[0] = FLASH_ReadByte(0x9000);
        
            //DEBUG 
            if (TxData[0] == 0x11)
                FlashLEDs(3);
            CAN_SendData(&TxData[0]);
        }
        if ((u8)(_RxMessage.Data[1]) == 0x02){
        
            FLASH_Unlock(FLASH_MEMTYPE_PROG);
            FLASH_ProgramByte(0x9000, _RxMessage.Data[2]);
            FLASH_Lock(FLASH_MEMTYPE_PROG);
        
        }
    }
}

u8 CAN_SendData(u8 *Data0)
{
    //CanTxMsg TxMessage;
    u8 TransmitMailbox;
    int txcnt;
	
	/* transmit */
	// set the ID to the board ID -- base should never transmit a message with ID 0x0
    _TxMessage.StdId=(u16)(0x0000);
    _TxMessage.ExtId_28_24=0x00;
    _TxMessage.ExtId_23_16=G_BOARDID[3];
    _TxMessage.ExtId_15_8=G_BOARDID[2];
    _TxMessage.ExtId_7_0=G_BOARDID[1];
    _TxMessage.IDE=CAN_ID_EXT;
    _TxMessage.RTR=CAN_RTR_DATA;
    _TxMessage.DLC=8;
    _TxMessage.Data[0]=Data0[0];
    _TxMessage.Data[1]=Data0[1];
    _TxMessage.Data[2]=Data0[2];
    _TxMessage.Data[3]=Data0[3];
    _TxMessage.Data[4]=Data0[4];
    _TxMessage.Data[5]=Data0[5];
    _TxMessage.Data[6]=Data0[6];
    _TxMessage.Data[7]=Data0[7];

    TransmitMailbox = CAN_Transmit(&_TxMessage);
	

    txcnt = 0;
    while( ( CAN_TransmitStatus(TransmitMailbox) ) != CANTXOK )
    {

		txcnt++;
        // Exit after a given number of retries
        if(txcnt > 100000)
        {				
			
				switch(TransmitMailbox)
				{
				case 0:
					CAN_PSR = 0x00;
					break;
				case 1:
					CAN_PSR = 0x01;
					break;
				case 2:
					CAN_PSR = 0x05;
					break;
				default:
					CAN_PSR = 0x00;
					break;
				}
	
				/* abort transmission */
				CAN_MCSR |= CAN_MCSR_ABRQ;	
				CAN_PSR = 0x00;
				break;
        }	
    }

	// pull CAN_STBY high again...
    return TransmitMailbox;
}

void SetLEDs(u8 newData)
{
	GPIO_Write(LEDS_PORT, 0x00);
    GPIO_Write(LEDS_PORT, ~newData);
}

void FlashLEDs( unsigned int nTimes ){
	// flash the LEDs
	unsigned short i;
	
	for( i=0; i<nTimes; i++ ){
		SetLEDs(LED_RED_PIN);
		delay(10);
		SetLEDs(LED_GREEN_PIN);
		delay(10);
		SetLEDs(0x00);
		delay(10);
	}
}


